########################################################################
# Stopping boundaries for Bayesian decision-theoretic designs
########################################################################
library(mvtnorm)


# Calculate the exact type I error rate for given boundary values
calc_type_I_error_rate = function(c, n) {
  
  # c and n are length K vectors
  # c: vector of stopping boundaries (on the z-statistic scale)
  # n: vector, numbers of observations after which interim analyses are performed
  
  K = length(c)
  if (K != length(n)) stop("Lengths of c and n do not match!")
  
  # Covariance matrix for the vector of z-statistics
  Sigma = diag(K)
  for(i in 2:K) {
    for(j in 1:(i-1)) {
      Sigma[i, j] = Sigma[j, i] = sqrt(n[j] / n[i])
    }
  }
  
  return(1 - pmvnorm(upper = c, corr = Sigma))
  
}





Bound_BayesDT = function(mu, nusq, xi1, xi0, sigmasq, n,
                         approx_level = 2) {
  
  # mu: a scalar, prior mean of theta
  # nusq: a scalar, prior variance of theta
  # xi1: a length K vector, loss associated with stopping trial at each analysis
  # xi0: a scalar, loss associated with failing to reject H0 at the final analysis
  # sigmasq: a scalar, variance of the (normal) outcomes
  # n: a length K vector, numbers of observations after which interim analyses are performed
  # approx_level: an integer scalar, at which levels we approximate the loss function (instead of exact calculation)
  #   approx_level explained with an example: if approx_level = 2, we do exact calculations for 
  #   L_K(0, z) and L_{K-1}(0, z), but approximate L_{K-2}(0, z), ..., L_1(0, z)
  #   The reason is that, the calculation of L_j(0, z) involves (K-j+1) multiple integrals
  #   For small j this can be very time-consuming, thus for
  #   j <= K - approx_level we approximate L_j(0, z)
  
  K = length(n)
  
  if(length(xi1) == 1) {
    xi1 = rep(xi1, K)
  } else {
    if(length(xi1) != K) {
      stop("Lengths of n and xi1 do not match!")
    }
  }
  
  # stopping boundaries
  c = rep(NA, K)

  z_grid_lower = -10
  z_grid_upper = 10
  n_z_grid = 1000
  z_grid = seq(z_grid_lower, z_grid_upper, length.out = n_z_grid)
  
  L_1_grid = matrix(NA, K, n_z_grid)
  L_0_grid = matrix(NA, K, n_z_grid)
  
  
  L = function(j, psi, z, c, L_1_grid, L_0_grid) {
    
    mean = (mu / nusq + z * sqrt(n[j] / sigmasq)) / (1 / nusq + n[j] / sigmasq)
    sd = 1 / sqrt(1 / nusq + n[j] / sigmasq)
    
    if(psi == 1) {
      
      return(xi1[j] * pnorm(0, mean = mean, sd = sd))
      
    } else {
      
      if(j == K) {
        
        return(xi0 * (1 - pnorm(0, mean = mean, sd = sd)))
        
      } else {
        
        zz_mean = sqrt(n[j] / n[j+1]) * z +
          (n[j+1] - n[j]) * (mu / nusq + z * sqrt(n[j] / sigmasq)) / 
          (sqrt(n[j+1] * sigmasq) * (1 / nusq + n[j] / sigmasq))
        
        zz_sd = (n[j+1] - n[j]) * 
          sqrt(1 / (1 / nusq + n[j] / sigmasq) +
                 1 / ((n[j+1] - n[j]) / sigmasq)) / 
          sqrt(sigmasq * n[j+1])
        
        if((K - j) < approx_level) {
          
          if(is.na(c[j+1])) {
            
            stop("c_{j+1} must be pre-calculated!")
            
          } else {
            
            L_tilde_int0 = sapply(zz_mean, function(x) {
              integrate(function(zz) {L(j+1, 0, zz, c) * dnorm(zz, mean = x, sd = zz_sd)}, 
                        lower = -Inf, upper = c[j+1])})
            
            L_tilde_int0 = unlist(L_tilde_int0[1, ])
            
            L_tilde_int1 = sapply(zz_mean, function(x) {
              integrate(function(zz) {L(j+1, 1, zz, c) * dnorm(zz, mean = x, sd = zz_sd)}, 
                        lower = c[j+1], upper = Inf)})
            
            L_tilde_int1 = unlist(L_tilde_int1[1, ])
            
          }
          
          
        } else {
          
          if(is.na(c[j+1]) | sum(is.na(L_0_grid[j+1, ])) > 0 | sum(is.na(L_1_grid[j+1, ])) > 0) {
            
            stop("c_{j+1}, L_{j+1}(0, z_grid), and L_{j+1}(1, z_grid) must be pre-calculated!")
          
          } else {
            L_tilde_int0 = sapply(zz_mean, function(x) {
              integrate(function(zz) {
                approx(z_grid, L_0_grid[j+1, ], zz)$y * dnorm(zz, mean = x, sd = zz_sd)
              }, lower = z_grid_lower, upper = c[j+1])})
            
            L_tilde_int0 = unlist(L_tilde_int0[1, ])
            
            L_tilde_int1 = sapply(zz_mean, function(x) {
              integrate(function(zz) {
                approx(z_grid, L_1_grid[j+1, ], zz)$y * dnorm(zz, mean = x, sd = zz_sd)
              }, lower = c[j+1], upper = z_grid_upper)})
            
            L_tilde_int1 = unlist(L_tilde_int1[1, ])
          }
          
        }
        
        return(n[j+1] - n[j] + L_tilde_int0 + L_tilde_int1)
      }
      
      
    }
    
  }
  
  
  for (j in K:1) {
    
    
    L_j_uniroot = function(z) {
      L(j, 1, z, c, L_1_grid, L_0_grid) - L(j, 0, z, c, L_1_grid, L_0_grid)
    }
    
    c_j = uniroot(L_j_uniroot, c(0, 5))
    c[j] = c_j$root
    
    L_1_grid[j, ] = L(j, 1, z_grid, c, L_1_grid, L_0_grid)
    L_0_grid[j, ] = L(j, 0, z_grid, c, L_1_grid, L_0_grid)
    
  }
  
  return(list(c = c, z_grid = z_grid, 
              L_0_grid = L_0_grid, L_1_grid = L_1_grid))
}



result_BayesDT = Bound_BayesDT(mu = 0, nusq = 1, 
                               xi1 = rep(34890, 5), xi0 = 1000, 
                               sigmasq = 1, n = c(200, 400, 600, 800, 1000))
c_BayesDT = result_BayesDT$c
calc_type_I_error_rate(c_BayesDT, c(200, 400, 600, 800, 1000))



##################################################################
# Investigators C and D's example: different stopping boundaries
#  after 200 observations according to whether there is another 
#  interim analysis after 300 observations
##################################################################

result_BayesDT_C = Bound_BayesDT(mu = 0, nusq = 1, 
                                 xi1 = rep(19*400, 3), xi0 = 400, 
                                 sigmasq = 1, n = c(200, 300, 400),
                                 approx_level = 3)

result_BayesDT_D = Bound_BayesDT(mu = 0, nusq = 1, 
                                 xi1 = rep(19*400, 2), xi0 = 400, 
                                 sigmasq = 1, n = c(200, 400),
                                 approx_level = 2)

result_BayesDT_C$c
result_BayesDT_D$c

# plot C and D's loss functions
plot(result_BayesDT_C$z_grid, result_BayesDT_C$L_1_grid[1, ], type = "l", col = "red", 
     xlim = c(1.6, 2.0), ylim = c(175, 425))
points(result_BayesDT_C$z_grid, result_BayesDT_C$L_0_grid[1, ], type = "l", col = "green")
points(result_BayesDT_D$z_grid, result_BayesDT_D$L_0_grid[1, ], type = "l", col = "blue")
