########################################################################
# Operating characteristics of a Bayesian sequential design
#   based on posterior probabilities
########################################################################

OC_BayesPP = function(K, g, mu0, nu0, mu, nu, 
  sigma = 1, gamma = 0.95, ntrial = 1000) {
  
  # All arguments, K, g, mu0, nu0, mu, nu, sigma, gamma, and ntrial, are scalars
  # K: number of (interim and final) analyses
  # g: group size. Assume patients are enrolled in groups of equal size g (So maximum sample size is K*g)
  # mu0, nu0: population mean and standard deviation for the actual distribution of theta in an infinite series of trials
  # mu, nu: prior mean and standard deviation for theta in the Bayesian design
  # sigma: variance of the outcomes in the sampling model
  # gamma: posterior probability threshold for early stopping, i.e., stop when Pr(theta > 0 | data) > gamma
  # ntrial: number of hypothetical trials to simulate

  # True distribution of theta: N(mu0, nu0^2)
  theta_all = rnorm(ntrial, mean = mu0, sd = nu0)
  rej_all = rep(NA, ntrial)
  coverage_all = rep(NA, ntrial)
  sample_size_all = rep(NA, ntrial)
  post_mean_all = rep(NA, ntrial)
  
  for (i in 1:ntrial) {
    N = K * g
    theta = theta_all[i]
    
    # data for all patients
    y = rnorm(N, mean = theta, sd = sigma)
    y_cumsum = cumsum(y)
    
    # z-statistics at interim analyses
    n_j = (1:K) * g
    z = y_cumsum[n_j] / (sigma * sqrt(n_j))
    
    # stopping boundaries
    c = qnorm(gamma) * sqrt(1 + sigma^2 / (n_j * nu^2)) - 
      mu * sigma / (sqrt(n_j) * nu^2)
    
    t = min(which(z > c))
    
    if(t == Inf) {
      rej_all[i] = 0
      sample_size_all[i] = N
      t = K
    } else {
      rej_all[i] = 1
      sample_size_all[i] = t * g
    }
    
    n_t = t * g
    post_mean = (mu / nu^2 + y_cumsum[n_t] / sigma^2) / (1 / nu^2 + n_t / sigma^2)
    post_sd = sqrt(1 / (1 / nu^2 + n_t / sigma^2))
    CI_lower = qnorm(0.025, mean = post_mean, sd = post_sd)
    CI_upper = qnorm(0.975, mean = post_mean, sd = post_sd)
    
    post_mean_all[i] = post_mean
    
    if (theta >= CI_lower & theta <= CI_upper) {
      coverage_all[i] = 1
    } else {
      coverage_all[i] = 0
    }
    
  }
  
  return(list(theta_all = theta_all,
              rej_all = rej_all,
              coverage_all = coverage_all,
              post_mean_all = post_mean_all,
              sample_size_all = sample_size_all))
  
}


N = 1000
K = 1000

result = OC_BayesPP(K = K, g = N/K, 
                    mu0 = 0, nu0 = 0.1, 
                    mu = 0, nu = 0.1, 
                    gamma = 0.95, ntrial = 10000)

theta_all = result$theta_all
rej_all = result$rej_all
coverage_all = result$coverage_all
post_mean_all = result$post_mean_all
sample_size_all = result$sample_size_all
      

FDR = length(theta_all[rej_all == 1 & theta_all <= 0]) / 
  length(theta_all[rej_all == 1])
FPR = length(theta_all[rej_all == 1 & theta_all <= 0]) / 
  length(theta_all[theta_all <= 0])
FOR = length(theta_all[rej_all == 0 & theta_all > 0]) / 
  length(theta_all[rej_all == 0])
FNR = length(theta_all[rej_all == 0 & theta_all > 0]) / 
  length(theta_all[theta_all > 0])
coverage = mean(coverage_all)
bias = mean(post_mean_all - theta_all)

OC = c(FDR, FPR, FOR, FNR, coverage, bias)
names(OC) = c("FDR", "FPR", "FOR", "FNR", "coverage", "bias")
round(OC, 3)











########################################################################
# Operating characteristics under 72 simulation scenarios
#   different combinations of nu0, nu, and K
########################################################################

N = 1000
ntrial = 10000

nu0_allscns = c(0.1, 0.5, 1)
nu_allscns = c(0.1, 0.5, 1, 10)
K_allscns = c(1, 2, 5, 10, 100, 1000)

FDR_allscns = array(0, c(3, 4, 6))
FPR_allscns = array(0, c(3, 4, 6))
FOR_allscns = array(0, c(3, 4, 6))
FNR_allscns = array(0, c(3, 4, 6))
coverage_allscns = array(0, c(3, 4, 6))
bias_allscns = array(0, c(3, 4, 6))

for(i in 1:3) {
  for(j in 1:4) {
    for(k in 1:6) {
      cat("Scenario ", i, " ", j, " ", k, "finished.\n")
      result = OC_BayesPP(K = K_allscns[k], g = N/K_allscns[k], 
                          mu0 = 0, nu0 = nu0_allscns[i], 
                          mu = 0, nu = nu_allscns[j], 
                          gamma = 0.95, ntrial = ntrial)
      theta_all = result$theta_all
      rej_all = result$rej_all
      coverage_all = result$coverage_all
      post_mean_all = result$post_mean_all
      sample_size_all = result$sample_size_all
      
      
      FDR_allscns[i, j, k] = length(theta_all[rej_all == 1 & theta_all <= 0]) / 
        length(theta_all[rej_all == 1])
      FPR_allscns[i, j, k] = length(theta_all[rej_all == 1 & theta_all <= 0]) / 
        length(theta_all[theta_all <= 0])
      FOR_allscns[i, j, k] = length(theta_all[rej_all == 0 & theta_all > 0]) / 
        length(theta_all[rej_all == 0])
      FNR_allscns[i, j, k] = length(theta_all[rej_all == 0 & theta_all > 0]) / 
        length(theta_all[theta_all > 0])
      coverage_allscns[i, j, k] = mean(coverage_all)
      bias_allscns[i, j, k] = mean(post_mean_all - theta_all)
    }
  }
}


for(i in 1:3) {
  for(k in 1:6) {
    string1 = c(round(FDR_allscns[i, , k] * 100, 1), 
                round(FPR_allscns[i, , k] * 100, 1),
                round(coverage_allscns[i, , k] * 100, 1))
    string2 = paste(sprintf("%.1f", string1), collapse = " & ")
    string3 = paste0(K_allscns[k], " & ", string2, " \\\\\n")
    cat(string3)
  }
}
