########################################################################
# Stopping boundaries for various frequentist and Bayesian designs
# (Bayesian decision-theoretic designs excluded; refer to a separate file)
########################################################################

library(gsDesign)
library(mvtnorm)


# Calculate the exact type I error rate for given boundary values
calc_type_I_error_rate = function(c, n) {
  
  # c and n are length K vectors
  # c: vector of stopping boundaries (on the z-statistic scale)
  # n: vector, numbers of observations after which interim analyses are performed
  
  K = length(c)
  if (K != length(n)) stop("Lengths of c and n do not match!")
  
  # Covariance matrix for the vector of z-statistics
  Sigma = diag(K)
  for(i in 2:K) {
    for(j in 1:(i-1)) {
      Sigma[i, j] = Sigma[j, i] = sqrt(n[j] / n[i])
    }
  }
  
  return(1 - pmvnorm(upper = c, corr = Sigma))
  
}

# If all tests performed at a nominal level of 0.05
# the type I error rate inflates as the number of analyses increases
calc_type_I_error_rate(rep(qnorm(0.95), 2), c(500, 1000))
calc_type_I_error_rate(rep(qnorm(0.95), 5), c(200, 400, 600, 800, 1000))
calc_type_I_error_rate(rep(qnorm(0.95), 10), seq(0, 1000, length.out = 11)[-1])
calc_type_I_error_rate(rep(qnorm(0.95), 100), seq(0, 1000, length.out = 101)[-1])





# Boundary values based on stochastically curtailed procedure (using conditional power)
Bound_SCP = function(eta, gamma, n) {
  # eta: a number
  # gamma: a number
  K = length(n)
  return(qnorm(eta, lower.tail = FALSE) * sqrt(n[K] / n) + 
           qnorm(1 - gamma, lower.tail = FALSE) * sqrt((n[K] - n) / n))
}

# Boundary values based on posterior probability (of theta > 0)
Bound_BayesPP = function(mu, nusq, sigmasq, gamma, n) {
  
  # gamma: a vector
  K = length(n)
  
  return(qnorm(gamma) * sqrt(1 + sigmasq / (n* nusq)) - 
           mu * sqrt(sigmasq) / (sqrt(n) * nusq))
}


# Boundary values based on posterior predictive probability (of success)
Bound_BayesPPP = function(mu, nusq, sigmasq, eta, gamma, n) {
  # gamma: a vector of length K
  K = length(n)
  
  denominator = sqrt(n[-K]) * sqrt(sigmasq) / (n[K] - n[-K]) +
    sqrt(n[-K]) / (sqrt(sigmasq) / nusq + n[-K] / sqrt(sigmasq))
  
  numerator = (qnorm(eta, lower.tail = FALSE) * 
                 sqrt(1 / nusq + n[K] / sigmasq) -
                 mu / nusq) / 
    ((n[K] - n[-K]) / sigmasq) - 
    mu / nusq / (1 / nusq + n[-K] / sigmasq) +
    qnorm(1 - gamma[-K], lower.tail = FALSE) *
    sqrt(1 / (1 / nusq + n[-K] / sigmasq) + 
           1 / ((n[K] - n[-K]) / sigmasq))
  
  bound = c(numerator / denominator, 
            qnorm(eta, lower.tail = FALSE) * 
              sqrt(1 + sigmasq / (n[K] * nusq)) - 
              mu * sqrt(sigmasq) / (sqrt(n[K]) * nusq))
  
  return(bound)
}

# Boundary values based on Pocock's procedure
c_P = gsDesign(k = 5, test.type = 1, alpha = .05, sfu = "Pocock")$upper$bound
# # Boundary values based on O'Brien-Fleming's procedure
c_OF = gsDesign(k = 5, test.type = 1, alpha = .05, sfu = "OF")$upper$bound
c_Spending_P = gsDesign(k = 5, test.type = 1, alpha = .05, sfu = sfLDPocock)$upper$bound
c_Spending_OF = gsDesign(k = 5, test.type = 1, alpha = .05, sfu = sfLDOF)$upper$bound
c_Spending_Power = gsDesign(k = 5, test.type = 1, alpha = .05, sfu = sfPower, sfupar = 1)$upper$bound
c_SCP = Bound_SCP(eta = 0.049, gamma = 0.8, n = c(200, 400, 600, 800, 1000))
c_BayesPP1 = Bound_BayesPP(mu = 0, nusq = 0.054^2, sigmasq = 1, gamma = rep(0.95, 5), 
                           n = c(200, 400, 600, 800, 1000))
c_BayesPP2 = Bound_BayesPP(mu = 0, nusq = 1, sigmasq = 1, gamma = rep(0.983, 5), 
                           n = c(200, 400, 600, 800, 1000))
c_BayesPPP = Bound_BayesPPP(mu = 0, nusq = 0.063^2, sigmasq = 1, 
                            eta = 0.05, gamma = rep(0.8, 5), 
                            n = c(200, 400, 600, 800, 1000))



# Verify that the calculated stopping boundaries lead to a type I error rate of 0.05
calc_type_I_error_rate(c_P, c(200, 400, 600, 800, 1000))
calc_type_I_error_rate(c_OF, c(200, 400, 600, 800, 1000))
calc_type_I_error_rate(c_Spending_Power, c(200, 400, 600, 800, 1000))
calc_type_I_error_rate(c_SCP, c(200, 400, 600, 800, 1000))
calc_type_I_error_rate(c_BayesPP1, c(200, 400, 600, 800, 1000))
calc_type_I_error_rate(c_BayesPP2, c(200, 400, 600, 800, 1000))
calc_type_I_error_rate(c_BayesPPP, c(200, 400, 600, 800, 1000))
